@@ -6,7 +6,7 @@ sub _getstash { \%{"$_[0]::"} }
 use strict;
 use warnings FATAL => 'all';
 
-our $VERSION = '1.002005'; # 1.2.5
+our $VERSION = '1.003000'; # 1.3.0
 $VERSION = eval $VERSION;
 
 our %INFO;
@@ -44,6 +44,7 @@ sub import {
   strict->import;
   warnings->import(FATAL => 'all');
   return if $INFO{$target}; # already exported into this package
+  $INFO{$target}{is_role} = 1;
   # get symbol table reference
   my $stash = _getstash($target);
   # install before/after/around subs
@@ -64,28 +65,33 @@ sub import {
   };
   # grab all *non-constant* (stash slot is not a scalarref) subs present
   # in the symbol table and store their refaddrs (no need to forcibly
-  # inflate constant subs into real subs) - also add '' to here (this
-  # is used later) with a map to the coderefs in case of copying or re-use
-  my @not_methods = ('', map { *$_{CODE}||() } grep !ref($_), values %$stash);
+  # inflate constant subs into real subs) with a map to the coderefs in
+  # case of copying or re-use
+  my @not_methods = (map { *$_{CODE}||() } grep !ref($_), values %$stash);
   @{$INFO{$target}{not_methods}={}}{@not_methods} = @not_methods;
   # a role does itself
   $APPLIED_TO{$target} = { $target => undef };
 }
 
+sub role_application_steps {
+  qw(_install_methods _check_requires _install_modifiers _copy_applied_list);
+}
+
 sub apply_single_role_to_package {
   my ($me, $to, $role) = @_;
 
   _load_module($role);
 
   die "This is apply_role_to_package" if ref($to);
-  die "${role} is not a Role::Tiny" unless my $info = $INFO{$role};
+  die "${role} is not a Role::Tiny" unless $INFO{$role};
 
-  $me->_check_requires($to, $role, @{$info->{requires}||[]});
-
-  $me->_install_methods($to, $role);
-
-  $me->_install_modifiers($to, $info->{modifiers});
+  foreach my $step ($me->role_application_steps) {
+    $me->$step($to, $role);
+  }
+}
 
+sub _copy_applied_list {
+  my ($me, $to, $role) = @_;
   # copy our role list into the target's
   @{$APPLIED_TO{$to}||={}}{keys %{$APPLIED_TO{$role}}} = ();
 }
@@ -129,7 +135,8 @@ sub create_class_with_roles {
     require MRO::Compat;
   }
 
-  my %conflicts = %{$me->_composite_info_for(@roles)->{conflicts}};
+  my $composite_info = $me->_composite_info_for(@roles);
+  my %conflicts = %{$composite_info->{conflicts}};
   if (keys %conflicts) {
     my $fail = 
       join "\n",
@@ -143,15 +150,22 @@ sub create_class_with_roles {
 
   my @composable = map $me->_composable_package_for($_), reverse @roles;
 
-  *{_getglob("${new_name}::ISA")} = [ @composable, $superclass ];
-
-  my @info = map $INFO{$_}, @roles;
+  # some methods may not exist in the role, but get generated by
+  # _composable_package_for (Moose accessors via Moo).  filter out anything
+  # provided by the composable packages, excluding the subs we generated to
+  # make modifiers work.
+  my @requires = grep {
+    my $method = $_;
+    !grep $_->can($method) && !$COMPOSED{role}{$_}{modifiers_only}{$method},
+      @composable
+  } @{$composite_info->{requires}};
 
   $me->_check_requires(
-    $new_name, $compose_name,
-    do { my %h; @h{map @{$_->{requires}||[]}, @info} = (); keys %h }
+    $superclass, $compose_name, \@requires
   );
 
+  *{_getglob("${new_name}::ISA")} = [ @composable, $superclass ];
+
   @{$APPLIED_TO{$new_name}||={}}{
     map keys %{$APPLIED_TO{$_}}, @roles
   } = ();
@@ -171,7 +185,7 @@ sub apply_roles_to_package {
   return $me->apply_role_to_package($to, $roles[0]) if @roles == 1;
 
   my %conflicts = %{$me->_composite_info_for(@roles)->{conflicts}};
-  delete $conflicts{$_} for $me->_concrete_methods_of($to);
+  delete $conflicts{$_} for keys %{ $me->_concrete_methods_of($to) };
   if (keys %conflicts) {
     my $fail = 
       join "\n",
@@ -190,8 +204,27 @@ sub apply_roles_to_package {
     delete $INFO{$to}{methods}; # reset since we're about to add methods
   }
 
-  foreach my $role (@roles) {
-    $me->apply_single_role_to_package($to, $role);
+  # backcompat: allow subclasses to use apply_single_role_to_package
+  # to apply changes.  set a local var so ours does nothing.
+  our %BACKCOMPAT_HACK;
+  if($me ne __PACKAGE__
+      and exists $BACKCOMPAT_HACK{$me} ? $BACKCOMPAT_HACK{$me} :
+      $BACKCOMPAT_HACK{$me} =
+        $me->can('role_application_steps')
+          == \&role_application_steps
+        && $me->can('apply_single_role_to_package')
+          != \&apply_single_role_to_package
+  ) {
+    foreach my $role (@roles) {
+      $me->apply_single_role_to_package($to, $role);
+    }
+  }
+  else {
+    foreach my $step ($me->role_application_steps) {
+      foreach my $role (@roles) {
+        $me->$step($to, $role);
+      }
+    }
   }
   $APPLIED_TO{$to}{join('|',@roles)} = 1;
 }
@@ -207,8 +240,11 @@ sub _composite_info_for {
       my $this_methods = $me->_concrete_methods_of($role);
       $methods{$_}{$this_methods->{$_}} = $role for keys %$this_methods;
     }
+    my %requires;
+    @requires{map @{$INFO{$_}{requires}||[]}, @roles} = ();
+    delete $requires{$_} for keys %methods;
     delete $methods{$_} for grep keys(%{$methods{$_}}) == 1, keys %methods;
-    +{ conflicts => \%methods }
+    +{ conflicts => \%methods, requires => [keys %requires] }
   };
 }
 
@@ -218,6 +254,8 @@ sub _composable_package_for {
   return $composed_name if $COMPOSED{role}{$composed_name};
   $me->_install_methods($composed_name, $role);
   my $base_name = $composed_name.'::_BASE';
+  # force stash to exist so ->can doesn't complain
+  _getstash($base_name);
   # Not using _getglob, since setting @ISA via the typeglob breaks
   # inheritance on 5.10.0 if the stash has previously been accessed an
   # then a method called on the class (in that order!), which
@@ -225,9 +263,9 @@ sub _composable_package_for {
   { no strict 'refs'; @{"${composed_name}::ISA"} = ( $base_name ); }
   my $modifiers = $INFO{$role}{modifiers}||[];
   my @mod_base;
-  foreach my $modified (
-    do { my %h; @h{map $_->[1], @$modifiers} = (); keys %h }
-  ) {
+  my @modifiers = grep !$composed_name->can($_),
+    do { my %h; @h{map @{$_}[1..$#$_-1], @$modifiers} = (); keys %h };
+  foreach my $modified (@modifiers) {
     push @mod_base, "sub ${modified} { shift->next::method(\@_) }";
   }
   my $e;
@@ -237,13 +275,16 @@ sub _composable_package_for {
     $e = "Evaling failed: $@\nTrying to eval:\n${code}" if $@;
   }
   die $e if $e;
-  $me->_install_modifiers($composed_name, $modifiers);
-  $COMPOSED{role}{$composed_name} = 1;
+  $me->_install_modifiers($composed_name, $role);
+  $COMPOSED{role}{$composed_name} = {
+    modifiers_only => { map { $_ => 1 } @modifiers },
+  };
   return $composed_name;
 }
 
 sub _check_requires {
-  my ($me, $to, $name, @requires) = @_;
+  my ($me, $to, $name, $requires) = @_;
+  return unless my @requires = @{$requires||$INFO{$name}{requires}||[]};
   if (my @requires_fail = grep !$to->can($_), @requires) {
     # role -> role, add to requires, role -> class, error out
     if (my $to_info = $INFO{$to}) {
@@ -266,8 +307,7 @@ sub _concrete_methods_of {
     # grab all code entries that aren't in the not_methods list
     map {
       my $code = *{$stash->{$_}}{CODE};
-      # rely on the '' key we added in import for "no code here"
-      exists $not_methods->{$code||''} ? () : ($_ => $code)
+      ( ! $code or exists $not_methods->{$code} ) ? () : ($_ => $code)
     } grep !ref($stash->{$_}), keys %$stash
   };
 }
@@ -304,7 +344,8 @@ sub _install_methods {
 }
 
 sub _install_modifiers {
-  my ($me, $to, $modifiers) = @_;
+  my ($me, $to, $name) = @_;
+  return unless my $modifiers = $INFO{$name}{modifiers};
   if (my $info = $INFO{$to}) {
     push @{$info->{modifiers}}, @{$modifiers||[]};
   } else {
@@ -362,8 +403,15 @@ sub does_role {
   return 0;
 }
 
+sub is_role {
+  my ($me, $role) = @_;
+  return !!$INFO{$role};
+}
+
 1;
 
+=encoding utf-8
+
 =head1 NAME
 
 Role::Tiny - Roles. Like a nouvelle cuisine portion size slice of Moose.
@@ -511,7 +559,7 @@ will work for classes but to test a role, one must use ::does_role directly.
 
 Additionally, Role::Tiny will override the standard Perl C<DOES> method
 for your class. However, if C<any> class in your class' inheritance
-heirarchy provides C<DOES>, then Role::Tiny will not override it.
+hierarchy provides C<DOES>, then Role::Tiny will not override it.
 
 =head1 METHODS
 
@@ -537,6 +585,12 @@ resulting class.
 Creates a new class based on base, with the roles composed into it in order.
 New class is returned.
 
+=head2 is_role
+
+ Role::Tiny->is_role('Some::Role1')
+
+Returns true if the given package is a role.
+
 =head1 SEE ALSO
 
 L<Role::Tiny> is the attribute-less subset of L<Moo::Role>; L<Moo::Role> is
